@@ -79,7 +79,6 @@ NautilusFileUndoManagerState nautilus_file_undo_manager_get_state (void);
 void nautilus_file_undo_manager_undo (GtkWindow *parent_window);
 void nautilus_file_undo_manager_redo (GtkWindow *parent_window);
 
-void nautilus_file_undo_manager_push_flag (void);
-gboolean nautilus_file_undo_manager_pop_flag (void);
+gboolean nautilus_file_undo_manager_is_operating (void);
 
 #endif /* __NAUTILUS_FILE_UNDO_MANAGER_H__ */
