@@ -88,6 +88,10 @@ void nautilus_get_x_content_types_for_mount_async (GMount *mount,
 						   NautilusMountGetContent callback,
 						   GCancellable *cancellable,
 						   gpointer user_data);
+char * get_message_for_content_type (const char *content_type);
+char * get_message_for_two_content_types (const char * const *content_types);
+gboolean should_handle_content_type (const char *content_type);
+gboolean should_handle_content_types (const char * const *content_type);
 
 gboolean nautilus_file_selection_equal (GList *selection_a, GList *selection_b);
 
