@@ -26,7 +26,6 @@
 #include "nautilus-file.h"
 #include "nautilus-file-private.h"
 #include "nautilus-file-utilities.h"
-#include "nautilus-global-preferences.h"
 #include "nautilus-search-provider.h"
 #include "nautilus-search-engine.h"
 #include "nautilus-search-engine-model.h"
@@ -66,6 +65,8 @@ struct NautilusSearchDirectoryDetails {
 	GList *callback_list;
 	GList *pending_callback_list;
 
+        GBinding *binding;
+
 	NautilusDirectory *base_model;
 };
 
@@ -157,7 +158,6 @@ start_search (NautilusSearchDirectory *search)
 {
 	NautilusSearchEngineModel *model_provider;
 	NautilusSearchEngineSimple *simple_provider;
-        NautilusFile *directory_as_file;
 	gboolean recursive;
 
 	if (!search->details->query) {
@@ -184,12 +184,7 @@ start_search (NautilusSearchDirectory *search)
 	nautilus_search_engine_model_set_model (model_provider, search->details->base_model);
 
 	simple_provider = nautilus_search_engine_get_simple_provider (search->details->engine);
-        directory_as_file = nautilus_directory_get_corresponding_file (search->details->base_model);
-        /* It's too slow to search recursively on locations that are on the network. So
-         * for that case, just search in the current directory */
-	recursive = g_settings_get_boolean (nautilus_preferences, "enable-recursive-search") &&
-                    !nautilus_directory_is_remote (search->details->base_model);
-        g_object_unref (directory_as_file);
+	recursive = nautilus_query_get_recursive (search->details->query);
 	g_object_set (simple_provider, "recursive", recursive, NULL);
 
 	reset_file_list (search);
@@ -892,19 +887,16 @@ nautilus_search_directory_set_base_model (NautilusSearchDirectory *search,
 	}
 
 	if (search->details->query != NULL) {
-		gchar *uri;
 		GFile *query_location, *model_location;
 		gboolean is_equal;
 
-		uri = nautilus_query_get_location (search->details->query);
-		query_location = g_file_new_for_uri (uri);
+                query_location = nautilus_query_get_location (search->details->query);
 		model_location = nautilus_directory_get_location (base_model);
 
 		is_equal = g_file_equal (model_location, query_location);
 
 		g_object_unref (model_location);
 		g_object_unref (query_location);
-		g_free (uri);
 
 		if (!is_equal) {
 			return;
@@ -945,13 +937,24 @@ nautilus_search_directory_set_query (NautilusSearchDirectory *search,
 				     NautilusQuery *query)
 {
 	NautilusFile *file;
+        NautilusQuery *old_query;
+
+        old_query = search->details->query;
 
 	if (search->details->query != query) {
-		g_object_ref (query);
-		g_clear_object (&search->details->query);
-		search->details->query = query;
+	        search->details->query = query;
+
+	        g_clear_pointer (&search->details->binding, g_binding_unbind);
+
+                if (query) {
+		        search->details->binding = g_object_bind_property (search->details->engine, "running",
+									   query, "searching",
+									   G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+                }
 
 		g_object_notify_by_pspec (G_OBJECT (search), properties[PROP_QUERY]);
+
+	        g_clear_object (&old_query);
 	}
 
 	file = nautilus_directory_get_existing_corresponding_file (NAUTILUS_DIRECTORY (search));
